use super::ValidatedOperation;

pub(crate) struct RefundDeleteAccountOperation {
    pub(crate) beneficiary_account: crate::models::AccountIdentifier,
}

impl super::ValidatedOperation for RefundDeleteAccountOperation {
    const OPERATION_TYPE: crate::models::OperationType =
        crate::models::OperationType::RefundDeleteAccount;

    fn into_operation(
        self,
        operation_identifier: crate::models::OperationIdentifier,
    ) -> crate::models::Operation {
        crate::models::Operation {
            operation_identifier,

            account: self.beneficiary_account,
            amount: None,
            metadata: None,

            related_operations: None,
            type_: Self::OPERATION_TYPE,
            status: None,
        }
    }
}

impl std::convert::TryFrom<crate::models::Operation> for RefundDeleteAccountOperation {
    type Error = crate::errors::ErrorKind;

    fn try_from(operation: crate::models::Operation) -> Result<Self, Self::Error> {
        Self::validate_operation_type(operation.type_)?;
        Ok(Self { beneficiary_account: operation.account })
    }
}
